<?php
namespace Interkassa\Model\PaymentType;
use \RS\Orm\Type;
use \Shop\Model\Orm\Transaction;

/**
* Способ оплаты - Interkassa
*/
class Interkassa extends \Shop\Model\PaymentType\AbstractType
{
    const
        API_URL     = "https://sci.interkassa.com/", //URL Api для взаимодействия
        PAYWAYS_URL = "https://api.interkassa.com/v1/paysystem-input-payway", //URL Api для получения полниго списка типов оплат
        IP_DIAPOZON = "85.10.225."; //Диапозон IP адресов интеркассы с которых должны приходить запросы
    
    /**
    * Возвращает название расчетного модуля (типа доставки)
    * 
    * @return string
    */
    function getTitle()
    {
        return t('Интеркасса');
    }
    
    /**
    * Возвращает описание типа оплаты. Возможен HTML
    * 
    * @return string
    */
    function getDescription()
    {
        return t('Оплата через агрегатор платежей "Интеркасса"');
    }
    
    /**
    * Возвращает идентификатор данного типа оплаты. (только англ. буквы)
    * 
    * @return string
    */
    function getShortName()
    {
        return 'interkassa';
    }
    
    /**
    * Отправка данных с помощью POST?
    * 
    */
    function isPostQuery()
    {
        return true;
    }
    
    /**
    * Возвращает ORM объект для генерации формы или null
    * 
    * @return \RS\Orm\FormObject | null
    */
    function getFormObject()
    {
        $properties = new \RS\Orm\PropertyIterator(array(
            'ik_co_id' => new Type\String(array(
                'maxLength' => 255,
                'description' => t('Checkout ID - индетификатор кассы'),
                
            )),
            'secret_key' => new Type\String(array(
                'description' => t('Секретный ключ'),
                'hint' => t('Указан на странице Вашей кассы'),
                'template' => '%interkassa%/form/payment/interkassa/secret_key.tpl'
            )),
            'test_key' => new Type\String(array(
                'description' => t('Тестовый ключ'),
                'hint' => t('Указан на странице Вашей кассы')
            )),
            'language' => new Type\String(array(
                'maxLength' => 5,
                'description' => t('Язык интерфейса'),
                'listFromArray' => array(array(
                    0    => t('Определяется Интеркассой'),
                    'ru' => t('Русский'),
                    'ua' => t('Украинский'),
                    'en' => t('Английский'),
                ))
            )),
            'ik_pw_via' => new Type\String(array(
                'maxLength' => 255,
                'description' => t('Тип оплаты:'),
                'default' => 0,
                'list' => array(array($this,'getPayways')),
            )),
            '__help__' => new Type\Mixed(array(
                'description' => t(''),
                'visible' => true,  
                'template' => '%interkassa%/form/payment/interkassa/help.tpl'
            )), 
        ));
        
        return new \RS\Orm\FormObject($properties);
    }
    
    
    /**
    * Возвращает true, если данный тип поддерживает проведение платежа через интернет
    * 
    * @return bool
    */
    function canOnlinePay()
    {
        return true;
    }
    
    /**
    * Возвращает URL для перехода на сайт сервиса оплаты
    * 
    * @param Transaction $transaction - ORM объект транзакции
    * @return string
    */
    function getPayUrl(\Shop\Model\Orm\Transaction $transaction)
    {
        
        $order      = $transaction->getOrder(); //Данные о заказе
        /**
        * @var mixed
        */
        $user       = $order->getUser(); //Пользователь который должен оплатить
        
        $inv_id     = $transaction->id;
        $out_summ   = round($transaction->cost, 2);
        $in_cur     = $this->getPaymentCurrency();
        $way        = $this->getOption('ik_pw_via',0); //Тип канала оплаты

        $params = array();
        $params['ik_co_id'] = $this->getOption('ik_co_id'); //ID кассы
        $params['ik_pm_no'] = $inv_id;
        $params['ik_cur']   = $in_cur;
        $params['ik_am']    = $out_summ;
        $params['ik_am_t']  = "invoice"; //Выбор способа оплаты будет на стороне интеркассы
        $params['ik_desc']  = t("Оплата заказа №").$order['order_num'];
        if ( $language = $this->getLanguage() ) {
           $params['ik_loc'] = $language; 
        }
        $params['ik_cli']  = $user['e_mail']; //Контакты покупателя
        
        //принудительно указываем метод post и url
        $router            = \RS\Router\Manager::obj();
        
        //Обработать process|payway|payways|payways_calc
        $params['ik_act'] = 'payways'; 
        if ($way){ //Если нужно отобразить все типы оплат
            $params['ik_act']    = 'payway';
            $params['ik_pw_via'] = $way; 
        }   
        
        $params['ik_int']  = 'web';     //Формат ответов json|web
        $params['ik_sign'] = $this->getParamsSign($params); 
        
        $this->addPostParams($params); //Добавляем параметры для POST запроса
        
        return self::API_URL; //url пост запроса
    }
    

    
    /**
    * Получает все варианты оплаты 
    * 
    * @param string $checkout_id - id кассы
    */
    function getPayways()
    {
        
        $params = array();
        
        $params['ik_co_id'] = $this->getOption('ik_co_id','');
        $params['ik_pm_no'] = 'PAYWAYS_1';
        $params['ik_am']    = 1;
        $params['ik_desc']  = 'Look up my payways';
        $params['ik_act']   = 'payways';
        $params['ik_int']   = 'json';
        $params['ik_sign']  = $this->getParamsSign($params);
        
        // Create a stream
        $opts = array(
            'http'=>array(
                'method'=>"GET",
            )
        );
        
        $context = stream_context_create($opts);
        $params = http_build_query($params);
        // Получим оплаты, которые привязаны к кассе
        $data = json_decode(file_get_contents(self::API_URL."?".$params, false, $context));

        // Подготовим массив
        $payways = array(0 => t('-Все типы оплат-'));
        
        if (isset($data->resultMsg) && $data->resultMsg=="Success"){ //Если всё прошло успешно и мы получили типы путей оплаты
            foreach($data->resultData->paywaySet as $way){
                $payways[$way->als] = mb_strtoupper($way->ser)." - ".mb_strtoupper($way->curAls);
            }
        }
        asort($payways);

        return $payways;
    }
    
    /**
    * Получает возможные пути оплаты для привязанной к вам кассы
    * 
    * @param array|string $data - id кассы или массив параметров, среди которых, должен быть ключ "checkout_id"
    */
    public static function staticGetPaywaysByCheckoutID($data)
    {
        $_this = new self();
        
        $_this->setOption('secret_key',$data['secret_key']);
        $_this->setOption('ik_co_id',$data['checkout_id']);
        $data = $_this->getPayways();
        
        $payways = array();
        foreach ($data as $key=>$value){
           $payways[] = array(
              'key' => $key,  
              'value' => $value,  
           ); 
        }
        
        return array('list' => $payways);
    }
    
    /**
    * Получает нужную подпись для разных режимов
    * 
    * @param boolean $test - флаг, что нужно использовать тестовый ключ
    */
    private function getRightSign($test = false)
    {
        if ($test && ($this->getOption('ik_pw_via',false) == "test_interkassa_test_xts")){
           file_put_contents(__DIR__.'/file.txt',date('Y-m-d H:i:s')."\n"."345435\n",FILE_APPEND);  
           return $this->getOption('test_key',''); 
        }
        return $this->getOption('secret_key','');
    }
    
    /**
    * Получает подпись для платежа формируемая по правилам Интеркассы
    * 
    * @param array $params - массив параметров
    * @param boolean $test - флаг, что нужно использовать тестовый ключ
    */
    private function getParamsSign( $params, $test = false )
    {
        unset($params['ik_sign']); //Удаляем из данных строку подписи 
        ksort($params, SORT_STRING); //Сортируем по ключам в алфовитном порядке
        array_push($params, $this->getRightSign($test)); //Конкатинируем символом ":"
        return base64_encode(md5(implode($params, ":"), true)); //MD5 в бинарном виде в кодированном base64
    }
    
    /**
    * Получает язык в котором будет представлен интерфейс Интеркассы
    */
    private function getLanguage()
    {
       return $this->getOption('language',0) ? $this->getOption('language',0) : false; 
    }
    
    /**
    * Проверяет диапозон IP адреса. Проверяет IP адреса интеркассы
    * 
    * @return boolean
    */ 
    private function checkIPsDiapozon()
    {
       $ip = $_SERVER['REMOTE_ADDR']; 
       if (stripos($ip, self::IP_DIAPOZON)===false){
          return false; 
       } 
       return true;
    }
    
    /**
    * Получает трех символьный код базовой валюты в которой ведётся оплата
    * 
    */
    private function getPaymentCurrency()
    {
       /**
       * @var \Catalog\Model\Orm\Currency
       */
       $currency = \RS\Orm\Request::make()
                        ->from(new \Catalog\Model\Orm\Currency())
                        ->where(array(
                           'public'  => 1,
                           'is_base'  => 1,
                        ))
                        ->object(); 
       return $currency ? $currency->title : false;
    }
    
    
    
    /**
    * Проверяем подпись запроса
    * 
    * @param string $sign - подпись запроса
    */
    private function checkSign($sign, $test = false)
    {
        $ik = array();
        foreach ($_REQUEST as $key=>$value){
           if (stripos('ik_')!==false){
              $ik[$key] = $value;  
           }
        }
        $my_sign = $this->getParamsSign($ik, $test); //Получаем нами сформированную подпись
        // Проверка корректности подписи
        return $my_sign == $sign;
    }
    
    /**
    * Проверяет основные параметры приходящие от интеркассы сравнивая с теми, что уснавлены в настройках системы
    * 
    * @param \Shop\Model\Orm\Transaction $transaction - объект транзакции
    * @param \RS\Http\Request $request - объект запросов
    */
    private function checkMainParams(\Shop\Model\Orm\Transaction $transaction, \RS\Http\Request $request)
    {
        $ik_co_id  = $request->request('ik_co_id',TYPE_STRING,'');
        $ik_am     = $request->request('ik_am',TYPE_STRING,'');
        $ik_inv_st = $request->request('ik_inv_st',TYPE_STRING,'');
        $ik_sign   = $request->request('ik_sign',TYPE_STRING,'');
        
        if ( $ik_co_id != $this->getOption('ik_co_id','') ) {
            return 'ID кассы';
        }
        if ( $ik_am != round($transaction->cost, 2) ) {
            return 'Сумма платежа';
        }
        if (!in_array($ik_inv_st,array('process','success','fail','waitAccept'))) {
            return 'параметр ik_inv_st='.$ik_inv_st;
        }
        $ik = array();
        foreach ($_REQUEST as $key=>$value){
           if (stripos('ik_')!==false){
              $ik[$key] = $value;  
           }
        }
        $test = ($request->request('ik_pw_via',TYPE_STRING,'') == "test_interkassa_test_xts");
        if (!$this->checkSign($ik_sign, $test)){
            return 'Неправильная подпись. Параметр ik_sign='.$ik_sign." ".$this->getParamsSign($ik);
        }
        return true;
    }
    
    /**
    * Возвращает ID заказа исходя из REQUEST-параметров соотвествующего типа оплаты
    * Используется только для Online-платежей
    * 
    * @return mixed
    */
    function getTransactionIdFromRequest(\RS\Http\Request $request)
    {
        return $request->request('ik_pm_no', TYPE_INTEGER, false);
    }
    
    /**
    * Обработка запросов от интеркассы
    * 
    * @param \Shop\Model\Orm\Transaction $transaction - объект транзакции
    * @param \RS\Http\Request $request - объект запросов
    * @return string
    */
    function onResult(\Shop\Model\Orm\Transaction $transaction, \RS\Http\Request $request)
    {
        
        if (!$this->checkIPsDiapozon()){
           $exception = new \Shop\Model\PaymentType\ResultException(t('Неправильный диапозон IP адресов'));
           $exception->setResponse('Wrong IPs');
           throw $exception; 
        }
        
        if (!$this->checkMainParams($transaction, $request)){
           $exception = new \Shop\Model\PaymentType\ResultException(t('Главные параметры указаные в настройках интеркассы не прошли проверку'));
           $exception->setResponse('Wrong main params');
           throw $exception;  
        }
        
        //Смотрим текущий статус
        $status = $request->request('ik_inv_st',TYPE_STRING,0);
        
        switch($status){
            case "success":
                return 'OK'.$transaction->id;
                break;
            case "process":
                $exception = new \Shop\Model\PaymentType\ResultException(t('Неудачно совершённый платёж'));
                $exception->setResponse('Payment status progress');
                $exception->setUpdateTransaction(false);
                throw $exception;  
                break;
            case "waitAccept": //Если долгое ожидание проведения платежа, то пользователь перенаправляется к нам
                \RS\Application\Application::getInstance()->headers->addHeader('Location',$request->getSelfAbsoluteHost());
                break;
            case "fail":
            default:
               $exception = new \Shop\Model\PaymentType\ResultException(t('Неудачно совершённый платёж'));
               $exception->setResponse('Payment failed');
               throw $exception;   
               break;
        }
        
        return 'OK'.$transaction->id;
    }
    
    /**
    * Вызывается при открытии страницы неуспешного проведения платежа 
    * Используется только для Online-платежей
    * 
    * @param \Shop\Model\Orm\Transaction $transaction
    * @param \RS\Http\Request $request
    * @return void 
    */
    function onFail(\Shop\Model\Orm\Transaction $transaction, \RS\Http\Request $request)
    {
        $transaction['status'] = $transaction::STATUS_FAIL;
        $transaction->update();
    }
    
}
